@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  */
 
 #include <config.h>
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 #include <syslog.h>
 #include <signal.h>
 #include <sys/wait.h>
@@ -109,13 +109,13 @@ gdm_display_check_loop (GdmDisplay *disp)
 	   * or just wait for the stupid malicious user to get bored
 	   * and go away */
 	  char *s = g_strdup_printf (C_(N_("The display server has been shut down "
-					   "about 6 times in the last 90 seconds, "
-					   "it is likely that something bad is "
-					   "going on.  I will wait for 2 minutes "
+					   "about 6 times in the last 90 seconds. "
+					   "It is likely that something bad is "
+					   "going on.  Waiting for 2 minutes "
 					   "before trying again on display %s.")),
 					disp->name);
 	  /* only display a dialog box if this is a local display */
-	  if (disp->type == TYPE_LOCAL ||
+	  if (disp->type == TYPE_STATIC ||
 	      disp->type == TYPE_FLEXI) {
 		  gdm_text_message_dialog (s);
 	  }
@@ -167,7 +167,7 @@ whack_old_slave (GdmDisplay *d, gboolean kill_connection)
     gboolean waitsleep = TRUE;
 
     if (kill_connection) {
-	    /* This should never happen I don't think, but just in case */
+	    /* This should never happen, but just in case */
 	    if (d->socket_conn != NULL) {
 		    GdmConnection *conn = d->socket_conn;
 		    d->socket_conn = NULL;
@@ -389,7 +389,7 @@ gdm_display_unmanage (GdmDisplay *d)
     whack_old_slave (d, TRUE /* kill_connection */);
     
     d->dispstat = DISPLAY_DEAD;
-    if (d->type != TYPE_LOCAL)
+    if (d->type != TYPE_STATIC)
 	gdm_display_dispose (d);
 
     gdm_debug ("gdm_display_unmanage: Display stopped");
@@ -409,12 +409,13 @@ count_session_limits (void)
 
 	for (li = displays; li != NULL; li = li->next) {
 		GdmDisplay *d = li->data;
-		if (d->type == TYPE_XDMCP) {
+		if (SERVER_IS_XDMCP (d)) {
 			if (d->dispstat == XDMCP_MANAGED)
 				xdmcp_sessions ++;
 			else if (d->dispstat == XDMCP_PENDING)
 				xdmcp_pending ++;
-		} else if (SERVER_IS_FLEXI (d)) {
+		}
+		if (SERVER_IS_FLEXI (d)) {
 			flexi_servers ++;
 		}
 	}
@@ -482,11 +483,19 @@ gdm_display_dispose (GdmDisplay *d)
     g_free (d->authfile_gdm);
     d->authfile_gdm = NULL;
 
-    if (d->xnest_temp_auth_file != NULL) {
-	    VE_IGNORE_EINTR (unlink (d->xnest_temp_auth_file));
+    if (d->type == TYPE_XDMCP_PROXY) {
+	    if (d->parent_auth_file != NULL) {
+		    VE_IGNORE_EINTR (unlink (d->parent_auth_file));
+	    }
+	    g_free (d->parent_auth_file);
+	    d->parent_auth_file = NULL;
     }
-    g_free (d->xnest_temp_auth_file);
-    d->xnest_temp_auth_file = NULL;
+
+    if (d->parent_temp_auth_file != NULL) {
+	    VE_IGNORE_EINTR (unlink (d->parent_temp_auth_file));
+    }
+    g_free (d->parent_temp_auth_file);
+    d->parent_temp_auth_file = NULL;
 
     if (d->auths) {
 	    gdm_auth_free_auth_list (d->auths);
@@ -514,11 +523,11 @@ gdm_display_dispose (GdmDisplay *d)
 	    gdm_choose_indirect_dispose_empty_id (d->indirect_id);
     d->indirect_id = 0;
 
-    g_free (d->xnest_disp);
-    d->xnest_disp = NULL;
+    g_free (d->parent_disp);
+    d->parent_disp = NULL;
 
-    g_free (d->xnest_auth_file);
-    d->xnest_auth_file = NULL;
+    g_free (d->parent_auth_file);
+    d->parent_auth_file = NULL;
 
     g_free (d->login);
     d->login = NULL;
