@@ -1,4 +1,4 @@
-/* GDM - The Gnome Display Manager
+/* GDM - The GNOME Display Manager
  * Copyright (C) 1998, 1999, 2000 Martin K. Petersen <mkp@mkp.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -17,7 +17,7 @@
  */
 
 #include "config.h"
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 #include <unistd.h>
 #include <signal.h>
 #include <sys/stat.h>
@@ -48,6 +48,10 @@ struct _GdmConnection {
 
 	GString *buffer;
 
+	int message_count;
+
+	gboolean nonblock;
+
 	int close_level; /* 0 - normal
 			    1 - no close, when called raise to 2
 			    2 - close was requested */
@@ -67,17 +71,19 @@ struct _GdmConnection {
 
 	GList *subconnections;
 	int n_subconnections;
+
+	GdmDisplay *disp;
 };
 
 static gboolean
-close_if_needed (GdmConnection *conn, GIOCondition cond)
+close_if_needed (GdmConnection *conn, GIOCondition cond, gboolean error)
 {
 	/* non-subconnections are never closed */
 	if (conn->parent == NULL)
 		return TRUE;
 
 	if (cond & G_IO_ERR ||
-	    cond & G_IO_HUP) {
+	    cond & G_IO_HUP || error) {
 		gdm_debug ("close_if_needed: Got HUP/ERR on %d", conn->fd);
 		conn->source = 0;
 		gdm_connection_close (conn);
@@ -97,11 +103,11 @@ gdm_connection_handler (GIOChannel *source,
 	size_t len;
 
 	if ( ! (cond & G_IO_IN)) 
-		return close_if_needed (conn, cond);
+		return close_if_needed (conn, cond, FALSE);
 
-	IGNORE_EINTR (len = read (conn->fd, buf, sizeof (buf) -1));
+	VE_IGNORE_EINTR (len = read (conn->fd, buf, sizeof (buf) -1));
 	if (len <= 0) {
-		return close_if_needed (conn, cond);
+		return close_if_needed (conn, cond, TRUE);
 	}
 
 	buf[len] = '\0';
@@ -115,8 +121,11 @@ gdm_connection_handler (GIOChannel *source,
 		     ve_string_empty (conn->buffer->str)))
 			/*ignore \r or empty lines*/
 			continue;
-		if (*p == '\n') {
+		if (*p == '\n' || 
+		    /* cut lines short at 4096 to prevent DoS attacks */
+		    conn->buffer->len > 4096) {
 			conn->close_level = 1;
+			conn->message_count ++;
 			conn->handler (conn, conn->buffer->str,
 				       conn->data);
 			if (conn->close_level == 2) {
@@ -132,7 +141,7 @@ gdm_connection_handler (GIOChannel *source,
 		}
 	}
 
-	return close_if_needed (conn, cond);
+	return close_if_needed (conn, cond, FALSE);
 }
 
 gboolean
@@ -147,6 +156,8 @@ gboolean
 gdm_connection_write (GdmConnection *conn, const char *str)
 {
 	int ret;
+	int save_errno;
+	int flags = 0;
 #ifndef MSG_NOSIGNAL
 	void (*old_handler)(int);
 #endif
@@ -154,18 +165,28 @@ gdm_connection_write (GdmConnection *conn, const char *str)
 	g_return_val_if_fail (conn != NULL, FALSE);
 	g_return_val_if_fail (str != NULL, FALSE);
 
-	if ( ! conn->writable)
+	if G_UNLIKELY ( ! conn->writable)
 		return FALSE;
 
+#ifdef MSG_DONTWAIT
+	if (conn->nonblock)
+		flags |= MSG_DONTWAIT;
+#endif
+
 #ifdef MSG_NOSIGNAL
-	IGNORE_EINTR (ret = send (conn->fd, str, strlen (str), MSG_NOSIGNAL));
+	VE_IGNORE_EINTR (ret = send (conn->fd, str, strlen (str), MSG_NOSIGNAL | flags));
+	save_errno = errno;
 #else
 	old_handler = signal (SIGPIPE, SIG_IGN);
-	IGNORE_EINTR (ret = send (conn->fd, str, strlen (str), 0));
+	VE_IGNORE_EINTR (ret = send (conn->fd, str, strlen (str), flags));
+	save_errno = errno;
 	signal (SIGPIPE, old_handler);
 #endif
 
-	if (ret < 0)
+	/* just so that 'signal' doesn't whack it */
+	errno = save_errno;
+
+	if G_UNLIKELY (ret < 0)
 		return FALSE;
 	else
 		return TRUE;
@@ -186,10 +207,10 @@ gdm_socket_handler (GIOChannel *source,
 	if ( ! (cond & G_IO_IN)) 
 		return TRUE;
 
-	IGNORE_EINTR (fd = accept (conn->fd,
+	VE_IGNORE_EINTR (fd = accept (conn->fd,
 				   (struct sockaddr *)&addr,
 				   &addr_size));
-	if (fd < 0) {
+	if G_UNLIKELY (fd < 0) {
 		gdm_debug ("gdm_socket_handler: Rejecting connection");
 		return TRUE;
 	}
@@ -197,6 +218,9 @@ gdm_socket_handler (GIOChannel *source,
 	gdm_debug ("gdm_socket_handler: Accepting new connection fd %d", fd);
 
 	newconn = g_new0 (GdmConnection, 1);
+	newconn->disp = NULL;
+	newconn->message_count = 0;
+	newconn->nonblock = conn->nonblock;
 	newconn->close_level = 0;
 	newconn->fd = fd;
 	newconn->writable = TRUE;
@@ -240,30 +264,57 @@ gdm_connection_open_unix (const char *sockname, mode_t mode)
 	GdmConnection *conn;
 	struct sockaddr_un addr;
 	int fd;
-
-	IGNORE_EINTR (unlink (sockname));
+	int try_again_attempts = 1000;
 
 	fd = socket (AF_UNIX, SOCK_STREAM, 0);
-	if (fd < 0) {
+	if G_UNLIKELY (fd < 0) {
 		gdm_error (_("%s: Could not make socket"),
 			   "gdm_connection_open_unix");
 		return NULL;
 	}
 
+try_again:
+	/* this is all for creating sockets in /tmp/ safely */
+	VE_IGNORE_EINTR (unlink (sockname));
+	if G_UNLIKELY (errno == EISDIR ||
+		       errno == EPERM) {
+		/* likely a directory, someone's playing tricks on us */
+		char *newname = NULL;
+		do {
+			g_free (newname);
+			newname = g_strdup_printf ("%s-renamed-%u",
+						   sockname,
+						   (guint)g_random_int ());
+		} while (access (newname, F_OK) == 0);
+		VE_IGNORE_EINTR (rename (sockname, newname));
+		if G_UNLIKELY (errno != 0)
+			try_again_attempts = 0;
+		g_free (newname);
+	} else if G_UNLIKELY (errno != 0) {
+		try_again_attempts = 0;
+	}
+
 	memset(&addr, 0, sizeof(addr));
 	strcpy (addr.sun_path, sockname);
 	addr.sun_family = AF_UNIX;
-	if (bind (fd,
-		  (struct sockaddr *) &addr, sizeof (addr)) < 0) {
+	if G_UNLIKELY (bind (fd,
+			     (struct sockaddr *) &addr, sizeof (addr)) < 0) {
 		gdm_error (_("%s: Could not bind socket"),
 			   "gdm_connection_open_unix");
-		IGNORE_EINTR (close (fd));
+		try_again_attempts --;
+		/* someone is being evil on us */
+		if (errno == EADDRINUSE && try_again_attempts >= 0)
+			goto try_again;
+		VE_IGNORE_EINTR (close (fd));
 		return NULL;
 	}
 
-	IGNORE_EINTR (chmod (sockname, mode));
+	VE_IGNORE_EINTR (chmod (sockname, mode));
 
 	conn = g_new0 (GdmConnection, 1);
+	conn->disp = NULL;
+	conn->message_count = 0;
+	conn->nonblock = FALSE;
 	conn->close_level = 0;
 	conn->fd = fd;
 	conn->writable = FALSE;
@@ -298,6 +349,9 @@ gdm_connection_open_fd (int fd)
 	g_return_val_if_fail (fd >= 0, NULL);
 
 	conn = g_new0 (GdmConnection, 1);
+	conn->disp = NULL;
+	conn->message_count = 0;
+	conn->nonblock = FALSE;
 	conn->close_level = 0;
 	conn->fd = fd;
 	conn->writable = FALSE;
@@ -328,9 +382,9 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 	GdmConnection *conn;
 	int fd;
 
-	IGNORE_EINTR (unlink (fifo));
+	VE_IGNORE_EINTR (unlink (fifo));
 
-	if (mkfifo (fifo, 0660) < 0) {
+	if G_UNLIKELY (mkfifo (fifo, 0660) < 0) {
 		gdm_error (_("%s: Could not make FIFO"),
 			   "gdm_connection_open_fifo");
 		return NULL;
@@ -338,15 +392,18 @@ gdm_connection_open_fifo (const char *fifo, mode_t mode)
 
 	fd = open (fifo, O_RDWR); /* Open with write to avoid EOF */
 
-	if (fd < 0) {
+	if G_UNLIKELY (fd < 0) {
 		gdm_error (_("%s: Could not open FIFO"),
 			   "gdm_connection_open_fifo");
 		return NULL;
 	}
 
-	IGNORE_EINTR (chmod (fifo, mode));
+	VE_IGNORE_EINTR (chmod (fifo, mode));
 
 	conn = g_new0 (GdmConnection, 1);
+	conn->disp = NULL;
+	conn->message_count = 0;
+	conn->nonblock = FALSE;
 	conn->close_level = 0;
 	conn->fd = fd;
 	conn->writable = FALSE;
@@ -453,7 +510,7 @@ gdm_connection_close (GdmConnection *conn)
 	}
 
 	if (conn->fd > 0) {
-		IGNORE_EINTR (close (conn->fd));
+		VE_IGNORE_EINTR (close (conn->fd));
 		conn->fd = -1;
 	}
 
@@ -495,4 +552,65 @@ gdm_connection_printf (GdmConnection *conn, const gchar *format, ...)
 	return ret;
 }
 
+int
+gdm_connection_get_message_count (GdmConnection *conn)
+{
+	g_return_val_if_fail (conn != NULL, -1);
+	return conn->message_count;
+}
+
+gboolean
+gdm_connection_get_nonblock (GdmConnection *conn)
+{
+	g_return_val_if_fail (conn != NULL, FALSE);
+	return conn->nonblock;
+}
+
+void
+gdm_connection_set_nonblock (GdmConnection *conn,
+			     gboolean nonblock)
+{
+	g_return_if_fail (conn != NULL);
+	conn->nonblock = nonblock;
+}
+
+GdmDisplay *
+gdm_connection_get_display (GdmConnection *conn)
+{
+	g_return_val_if_fail (conn != NULL, NULL);
+	return conn->disp;
+}
+
+void
+gdm_connection_set_display (GdmConnection *conn,
+			    GdmDisplay *disp)
+{
+	g_return_if_fail (conn != NULL);
+	conn->disp = disp;
+}
+
+void
+gdm_kill_subconnections_with_display (GdmConnection *conn,
+				      GdmDisplay *disp)
+{
+	GList *subs;
+
+	g_return_if_fail (conn != NULL);
+	g_return_if_fail (disp != NULL);
+
+	subs = conn->subconnections;
+	while (subs != NULL) {
+		GdmConnection *subcon = subs->data;
+		if (subcon->disp == disp) {
+			subcon->disp = NULL;
+			conn->subconnections =
+				g_list_remove (conn->subconnections, subcon);
+			gdm_connection_close (subcon);
+			subs = conn->subconnections;
+		} else {
+			subs = subs->next;
+		}
+	}
+}
+
 /* EOF */
